using System;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.DataBinding.BindingToMultipleSources
{
    [Example("Binding to multiple sources", "Data binding")]
    public partial class BindingToMultipleSourcesForm : UserControl
    {
        private readonly BindingList<Order> _orders;
        private readonly BindingList<Strategy> _strategies;
        private readonly Random _r = new Random();
        private string _filterExpression = string.Empty;

        public BindingToMultipleSourcesForm()
        {
            InitializeComponent();


            //Create and bind orders to grid1
            _orders = new BindingList<Order>();
            _orders.AddingNew += OnAddingNewOrder;

            BindingSource ordersSource1 = new BindingSource();
            ordersSource1.DataSource = _orders;
            grid1.DataSource = ordersSource1;
            bindingNavigator1.BindingSource = ordersSource1;


            //Create and bind strategies to grid2
            _strategies = new BindingList<Strategy>();
            _strategies.AddingNew += OnAddingNewStrategy;

            BindingSource ordersSource2 = new BindingSource();
            ordersSource2.DataSource = _strategies;
            grid2.DataSource = ordersSource2;
            bindingNavigator2.BindingSource = ordersSource2;

            //Bind the grid3 to both data sources
            DataSourceCollection dataSourceCollection = new DataSourceCollection();
            dataSourceCollection.GetList().Add(_orders);
            dataSourceCollection.GetList().Add(_strategies);
            grid3.DataSource = dataSourceCollection;

            grid3.Filter = new Filter(IsRowFiltered);
        }
        
        #region Event handlers

        void OnAddingNewStrategy(object sender, AddingNewEventArgs e)
        {
            Strategy strategy = new Strategy("Strategy #" + (_strategies.Count + 1));
            strategy.Price = _r.Next(1000, 5000) + _r.NextDouble();
            strategy.Quantity = _r.Next(1, 100);
            e.NewObject = strategy;
        }

        void OnAddingNewOrder(object sender, AddingNewEventArgs e)
        {
            Order order = new Order("Order #" + (_orders.Count + 1), "Instrument #" + _r.Next(100, 10000));
            order.Price = _r.Next(100, 500) + _r.NextDouble();
            order.Quantity = _r.Next(100, 10000);
            e.NewObject = order;
        }

        private void OnFilterTextChanged(object sender, EventArgs e)
        {
            _filterExpression = tbFilter.Text;
            _filterExpression = _filterExpression.Replace("*", ".*");
            _filterExpression = _filterExpression.Replace("?", ".");

            grid3.FilterRefresh();
        }

        #endregion

        #region Private methods

        private bool IsRowFiltered(Row row)
        {
            Cell cell = row["Name"];
            string text = cell != null ? cell.Text : string.Empty;
            if (!string.IsNullOrEmpty(_filterExpression) && cell != null && !string.IsNullOrEmpty(text))
            {
                try
                {
                    return !(Regex.IsMatch(cell.Text, "^" + _filterExpression, RegexOptions.Singleline | RegexOptions.IgnoreCase));
                }
                catch
                {
                    return false;
                }
            }
            return false;
        }

        #endregion

    }
}
