using System.ComponentModel;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Demo.Examples.DataBinding.BindingToMultipleSources
{
    public class Order : INotifyPropertyChanged
    {
        #region Fields

        private readonly string _name;
        private readonly string _instrument;
        private double _price;
        private long _quantity;

        #endregion

        public Order(string name, string instrument)
        {
            _name = name;
            _instrument = instrument;
        }

        public string Name
        {
            get { return _name; }
        }

        public string Instrument
        {
            get { return _instrument; }
        }

        [DoubleFormat(2)]
        public double Price
        {
            get { return _price; }
            set
            {
                if(_price != value)
                {
                    _price = value;
                    FirePropertyChanged("Price");                    
                }
            }
        }

        public long Quantity
        {
            get { return _quantity; }
            set
            {
                if(_quantity != value)
                {
                    _quantity = value;
                    FirePropertyChanged("Quantity");
                }
            }
        }

        private void FirePropertyChanged(string propertyName)
        {
            if(PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
