using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Editors;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    public class BulletEditor : UITypeEditorEx
    {
        public override bool GetPaintCellSupported()
        {
            return true;
        }

        public override void PaintCell(PaintCellEventArgs e)
        {
            //Do basic painting without text
            e.Text = string.Empty;
            e.PaintAll();
            e.Handled = true;

            //Draw the stars over the cell
            Rectangle bounds = e.Cell.VirtualBounds;
            int offset = (bounds.Width - 5 * Resources.star_grey.Width) / 2;
            bounds.X += Math.Max(0, offset);

            bounds.Width = Resources.star_grey.Width;
            for (int i = 0; i < 5; i++)
            {
                //Select image
                Image image = null;
                switch (i)
                {
                    case 0:
                        image = Resources.bullet_ball_glass_grey;
                        break;
                    case 1:
                        image = Resources.bullet_ball_glass_green;
                        break;
                    case 2:
                        image = Resources.bullet_ball_glass_blue;
                        break;
                    case 3:
                        image = Resources.bullet_ball_glass_yellow;
                        break;
                    case 4:
                        image = Resources.bullet_ball_glass_red;
                        break;
                }

                if (image != null)
                {
                    Rectangle r2 = bounds;
                    //Deflate the image
                    if (!Equals(i, e.Cell.Value))
                    {
                        r2.Inflate(-4, -4);    
                    }

                    e.Graphics.DrawImage(image, r2, new Rectangle(new Point(), image.Size), GraphicsUnit.Pixel);
                    bounds.X += bounds.Width;
                }
            }

            base.PaintCell(e);
        }

        public override StopEditReason EditCell(IGridEditorService service, Cell cell, StartEditReason reason)
        {
            StopEditReason stopReason = StopEditReason.Undefined;
            //Edit the cell if the user has clicked on the cell with the left button
            if (cell.Row != null && Equals(StartEditReason.LButtonClick, reason))
            {
                Point pt = cell.Row.Grid.PointToClient(Cursor.Position);
                Rectangle bounds = cell.VirtualBounds;
                int offset = (bounds.Width - 5 * Resources.star_grey.Width) / 2;
                bounds.X += Math.Max(0, offset);


                int imageWidth = Resources.star_grey.Width;
                if (bounds.Contains(pt) && imageWidth > 0)
                {
                    //Set a new rating to the data object
                    cell.Value = (pt.X - bounds.X) / imageWidth;
                    cell.Invalidate();
                }
                stopReason = StopEditReason.UserStop;
            }
            return stopReason;
        }

        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
        {
            return UITypeEditorEditStyle.None;
        }

        public override bool Navigable
        {
            get { return false; }
        }
    }
}