using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Editors;
using Dapfor.Net.Formats;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.SharedData
{
    [Example("Shared data", "Features")]
    public partial class SharedDataDemoForm : UserControl
    {
        public SharedDataDemoForm()
        {
            //Initialize the grid
            InitializeComponent();

            //grid1: set custom editors and formats
            grid1.Headers[0]["IntValue3"].Editor = new BulletComboEditor();
            grid1.Headers[0]["IntValue3"].Format = new BulletFormat();
            grid1.Headers[0]["BoolValue"].Editor = new CheckBoxEditor();
            grid1.Headers[0]["IntValue4"].Editor = new UpDownNumericEditor();

            //grid2: set custom editors and formats
            grid2.Headers[1]["IntValue1"].Editor = new TrackBarEditor(0, 99);
            grid2.Headers[1]["IntValue2"].Format = new HexFormat(true, true);
            grid2.Headers[1]["IntValue3"].Editor = new BulletEditor();
            grid2.Headers[1]["BoolValue"].Editor = new IconBoolEditor();
            grid2.Headers[1]["IntValue4"].Editor = new UpDownNumericEditor();
            
            PopulateGrids();

            grid2.Rows.ExpandAll();
        }

        private void PopulateGrids()
        {
            Random random = new Random();
            List<DataObject> dataObjects = new List<DataObject>();
            for(int i = 0; i < 8; ++i)
            {
                dataObjects.Add(new DataObject(string.Format("Id{0}", i), random));
            }

            //Populate the grid1
            grid1.DataSource = dataObjects;

            
            //Add root rows to the grid2
            Row root1 = grid2.Rows.Add(new RootDataObject("Node 0: To expand the node double click on the row."));
            Row root2 = grid2.Rows.Add(new RootDataObject("Node 1: To expand the node double click on the row."));

            //Add 4 children to the root1
            for(int i = 0; i < 4; ++i)
            {
                root1.Add(dataObjects[i]);
            }
            //Add 4 children to the root2
            for (int i = 4; i < dataObjects.Count; ++i)
            {
                root2.Add(dataObjects[i]);
            }
        }

        #region Event handlers

        private void OnPaintGroupRow(object sender, PaintGroupRowEventArgs e)
        {
            if (e.Column != null)
            {
                if(e.Column.Id == "IntValue3")
                {
                    switch ((int)e.Value)
                    {
                        case 0:
                            e.Image = Resources.bullet_ball_glass_grey;
                            break;
                        case 1:
                            e.Image = Resources.bullet_ball_glass_green;
                            break;
                        case 2:
                            e.Image = Resources.bullet_ball_glass_blue;
                            break;
                        case 3:
                            e.Image = Resources.bullet_ball_glass_yellow;
                            break;
                        case 4:
                            e.Image = Resources.bullet_ball_glass_red;
                            break;
                    }
                }
                else if (e.Column.Id == "BoolValue")
                {
                    e.Image = Equals(e.Value, true)
                                  ? Resources.check
                                  : Resources.fatal;
                }
            }
        }

        #endregion
    }
}