using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Editors;


namespace Dapfor.Net.Demo.Examples.Features.StandardEditors
{
    [Example("Standard editors", "Features")]
    public partial class StandardEditorsDemoForm : UserControl
    {
        private readonly Random _random = new Random();

        public StandardEditorsDemoForm()
        {
            InitializeComponent();

            //Set a specific alignment for the 'Color' column
            grid.Headers[0]["Id5"].Appearance.CellTextAlignment = ContentAlignment.MiddleLeft;
            grid.Headers[0]["Id5"].Appearance.CellTextSettings.Padding = new Padding(4, 0, 2, 0);
            grid.Headers[0].GroupingEnabled = true;
            
            //Set a checkBox editor for the first column
            grid.Headers[0][0].Editor = new CheckBoxEditor();

            PopulateGrid();

            //Initialize controls
            cbHotFrame.Checked = grid.EditInPlace.HotFrameEnabled;
            btnHotFrameColor.Color = grid.EditInPlace.HotFrameColor;
            cbEnabled.Checked = grid.EditInPlace.Enabled;
            cbShowDropButtons.Checked = grid.EditInPlace.AlwaysShowDropButtons;
            cbAdjust.Checked = grid.EditInPlace.AdjustAfterEdit;
            cbTextEditor.Checked = grid.EditInPlace.DropDownTextEditor;
        }

        private void PopulateGrid()
        {
            //Add 20 custom objects to the grid
            for (int i = 0; i < 20; ++i)
            {
                grid.Rows.Add(new DataObject(_random));
            }
        }

        private void OnHotFrameEnabled(object sender, EventArgs e)
        {
            grid.EditInPlace.HotFrameEnabled = cbHotFrame.Checked;
        }

        private void OnHotFrameColorChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.HotFrameColor = btnHotFrameColor.Color;
        }

        private void OnEnabledChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.Enabled = cbEnabled.Checked;
        }

        private void OnShowDropButtonsChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.AlwaysShowDropButtons = cbShowDropButtons.Checked;
        }

        private void OnAdjustChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.AdjustAfterEdit = cbAdjust.Checked;
        }

        private void OnTextEditorChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.DropDownTextEditor = cbTextEditor.Checked;
        }
    }
}
