using System;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.Tooltips
{
    [Example("Tooltips", "Features")]
    public partial class TooltipDemoForm : UserControl
    {
        public TooltipDemoForm()
        {
            InitializeComponent();

            //Populate the grid
            Random random = new Random();
            for (int i = 0; i < 10; ++i)
            {
                int[] values = new int[10];
                for (int j = 0; j < values.Length; ++j)
                {
                    values[j] = random.Next() % 1000;
                }
                grid.Rows.Add(values);
            }

            //Set initial values to controls
            btnTooltipColor.Color = grid.Tooltips.TooltipColor;
            btnCellFrameColor.Color = grid.Tooltips.CellFrameColor;
            cbEnableCellFrame.Checked = grid.Tooltips.CellFrameEnabled;
            cbEnableTooltip.Checked = grid.Tooltips.TooltipEnabled;

            OnCbErrorsCheckedChanged(this, EventArgs.Empty);

            grid.Select();
        }

        private void OnGridCellEnter(object sender, GridCellEventArgs e)
        {
            if (!string.IsNullOrEmpty(tbMessage.Text) && string.IsNullOrEmpty(e.Cell.TooltipMessage))
            {
                e.Cell.ShowTooltip(tbMessage.Text, TimeSpan.FromSeconds(10));
            }
        }

        private void OnCbBalloonCheckedChanged(object sender, EventArgs e)
        {
            grid.Tooltips.IsBalloon = cbBalloon.Checked;
        }

        private void OnCbErrorsCheckedChanged(object sender, EventArgs e)
        {
            //Set messages into random cells
            grid.Tooltips.ClearCellMessages();
            if (cbErrors.Checked)
            {
                if (grid.Rows.Count > 0 && grid.Headers[0].Count > 0)
                {
                    Random random = new Random();
                    for (int i = 0; i < 10; ++i)
                    {
                        Cell cell = grid.Rows[random.Next() % grid.Rows.Count][random.Next() % grid.Headers[0].Count];
                        cell.TooltipMessage = string.Format("Cell[{0},{1}]: Some tooltip message", cell.Row.VisibleIndex, cell.Column.VisibleIndex);
                    }
                }
            }
        }

        private void OnBtnTooltipColorColorChanged(object sender, EventArgs e)
        {
            grid.Tooltips.TooltipColor = btnTooltipColor.Color;
        }

        private void OnBtnCellFrameColorColorChanged(object sender, EventArgs e)
        {
            grid.Tooltips.CellFrameColor = btnCellFrameColor.Color;
        }

        private void OnCbEnableCellFrameCheckedChanged(object sender, EventArgs e)
        {
            grid.Tooltips.CellFrameEnabled = cbEnableCellFrame.Checked;
        }

        private void OnCbEnableTooltipCheckedChanged(object sender, EventArgs e)
        {
            grid.Tooltips.TooltipEnabled = cbEnableTooltip.Checked;
        }
    }
}