using System;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.ContentSearch
{
    [Example("Content search", "Miscellaneous")]
    public partial class ContentSearchDemoForm : UserControl
    {
        #region Fields

        private readonly Random random = new Random();

        private string regExpression;

        #endregion

        public ContentSearchDemoForm()
        {
            //Initialize the grid
            InitializeComponent();

            //Populate the grid
            const int count = 10000;
            for(int i = 0; i < count; ++i)
            {
                int[] data = new int[grid.Headers[0].Columns.Count];
                for (int j = 0; j < grid.Headers[0].Columns.Count; j++)
                {
                    data[j] = random.Next()%(count*data.Length / 10);
                }
                grid.Rows.Add(data);
            }
        }

        private void OnGridPaintCell(object sender, PaintCellEventArgs e)
        {
            if (IsMatch(e.Cell))
            {
                //Highlight the cell if it matches to the filter
                e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, Color.FromArgb(100, Color.Orange));
                e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, Color.FromArgb(100, Color.Orange));
            }
        }

        private void OnFilterUpdated(object sender, EventArgs e)
        {
            //Regular expression
            regExpression = tbFilter.Text;
            regExpression = regExpression.Replace("*", ".*");
            regExpression = regExpression.Replace("?", ".");

            grid.FocusedRow = null;
            grid.Headers[0].FocusedColumn = null;
            Cell cell = FindNext();
            if (cell != null)
            {
                cell.EnsureVisible();
            }

            grid.Invalidate(false);
        }

        private Cell FindNext()
        {
            Header header = grid.Headers[0];
            Row row = grid.FocusedRow;
            Column column = header.FocusedColumn;
            if(row != null && column != null)
            {
                if (column.VisibleIndex == header.VisibleColumns.Count - 1)
                {
                    //look from the next row
                    column = header.VisibleColumns[0];
                    row = row.NextVisible;
                }
                else
                {
                    //look from the next column
                    column = header.VisibleColumns[column.VisibleIndex + 1];
                }
            }
            else
            {
                //There is not focused cell. Take the top left cell in the grid
                row = grid.Rows[0];
                column = header.VisibleColumns[0];
            }

            Cell foundCell = null;
            if(row != null && column != null)
            {
                for (int i = row.VisibleIndex; i < grid.Rows.Count && foundCell == null; ++i)
                {
                    row = grid.Rows[i];
                    for (int j = column.VisibleIndex; j < header.VisibleColumns.Count; ++j)
                    {
                        column = header.VisibleColumns[j];

                        Cell cell = row[column.Index];
                        if (IsMatch(cell))
                        {
                            cell.Focused = true;
                            cell.EnsureVisible();
                            foundCell = cell;
                            break;
                        }
                    }
                    column = header.VisibleColumns[0];
                }
            }
            return foundCell;
        }

        private Cell FindPrev()
        {
            Header header = grid.Headers[0];
            Row row = grid.FocusedRow;
            Column column = header.FocusedColumn;
            if (row != null && column != null)
            {
                if (column.VisibleIndex == 0)
                {
                    //look from the prev row
                    column = header.VisibleColumns[header.VisibleColumns.Count - 1];
                    row = row.PrevVisible;
                }
                else
                {
                    //look from the prev column
                    column = header.VisibleColumns[column.VisibleIndex - 1];
                }
            }
            else
            {
                //There is not focused cell. Take the top left cell in the grid
                row = grid.Rows[grid.Rows.Count - 1];
                column = header.VisibleColumns[header.VisibleColumns.Count - 1];
            }

            Cell foundCell = null;
            if (row != null && column != null)
            {
                for (int i = row.VisibleIndex; i >= 0 && foundCell == null; --i)
                {
                    row = grid.Rows[i];
                    for (int j = column.VisibleIndex; j >= 0; --j)
                    {
                        column = header.VisibleColumns[j];

                        Cell cell = row[column.Index];
                        if (IsMatch(cell))
                        {
                            cell.Focused = true;
                            cell.EnsureVisible();
                            foundCell = cell;
                            break;
                        }
                    }
                    column = header.VisibleColumns[header.VisibleColumns.Count - 1];
                }
            }
            return foundCell;
        }

        private bool IsMatch(Cell cell)
        {
            if(!string.IsNullOrEmpty(regExpression) && cell != null && cell.Row != null && cell.Column != null)
            {
                try
                {
                    return (Regex.IsMatch(cell.Text, "^" + regExpression, RegexOptions.Singleline | RegexOptions.IgnoreCase));
                }
                catch 
                {
                    return false;
                }                
            }
            return false;
        }

        private void OnFindNext(object sender, EventArgs e)
        {
            Cell cell = FindNext();
            if(cell != null)
            {
                cell.EnsureVisible();
            }
        }

        private void OnFindPrev(object sender, EventArgs e)
        {
            Cell cell = FindPrev();
            if (cell != null)
            {
                cell.EnsureVisible();
            }
        }

        private void OnFindLast(object sender, EventArgs e)
        {
            Header header = grid.Headers[0];
            Cell cell = grid.Rows[grid.Rows.Count - 1][header.VisibleColumns[header.VisibleColumns.Count - 1].Index];
            cell.Focused = true;
            cell.EnsureVisible();
            if(!IsMatch(cell))
            {
                cell = FindPrev();
                if (cell != null)
                {
                    cell.Focused = true;
                    cell.EnsureVisible();
                }
            }
        }

        private void OnFindFirst(object sender, EventArgs e)
        {
            Cell cell = grid.Rows[0][0];
            cell.Focused = true;
            cell.EnsureVisible();
            if (!IsMatch(cell))
            {
                cell = FindNext();
                if (cell != null)
                {
                    cell.Focused = true;
                    cell.EnsureVisible();
                }
            }
        }
    }
}