using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Performance.RealtimeUpdates
{
    public partial class TestGrid : Grid
    {
        private Timer timer;

        private readonly Queue<double> timeMetrics = new Queue<double>();
        private int currentCount;
        private int lastCount;
        private readonly HighPerformanceTimer highPerfTimer;
        

        public TestGrid()
        {
            highPerfTimer = new HighPerformanceTimer();

            InitializeComponent();

            timer = new Timer();
            timer.Interval = 1000;
            timer.Tick += OnTick;
            timer.Start();
        }

        private void OnTick(object sender, EventArgs e)
        {
            lastCount = currentCount;
            currentCount = 0;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            currentCount++;

            //Compute time, passed in the OnPaint method
            highPerfTimer.Start();

            //Process painting in the base class
            base.OnPaint(e);
            
            highPerfTimer.Stop();
            
            timeMetrics.Enqueue(highPerfTimer.Duration);

            //Keep only last 10 metrics - it is enough to show average time, passed in the OnPaint method
            if (timeMetrics.Count > 10)
            {
                timeMetrics.Dequeue();
            }
        }

        public double AverageTime
        {
            get
            {
                //Compute average time
                double avgDuration = 0;
                foreach (double duration in timeMetrics)
                {
                    avgDuration += duration;
                }

                return timeMetrics.Count > 0 ? avgDuration / timeMetrics.Count : 0;
            }
        }

        public double AverageCallsPerSecond
        {
            get
            {
                return lastCount;
            }
        }

    }
}