using System.ComponentModel;
using System.Runtime.InteropServices;

namespace Dapfor.Net.Demo.Internal
{
    internal class HighPerformanceTimer
    {
        private readonly long freq;
        private long startTime;
        private long stopTime;

        // Constructor
        public HighPerformanceTimer()
        {
            startTime = 0;
            stopTime = 0;

            if (QueryPerformanceFrequency(out freq) == false)
            {
                // high-performance counter not supported
                throw new Win32Exception();
            }
        }

        // Returns the duration of the timer (in milliseconds)
        public double Duration
        {
            get { return (double) 1000*(stopTime - startTime)/freq; }
        }

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceCounter(
            out long lpPerformanceCount);

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(
            out long lpFrequency);

        // Start the timer
        public void Start()
        {
            QueryPerformanceCounter(out startTime);
        }

        // Stop the timer
        public void Stop()
        {
            QueryPerformanceCounter(out stopTime);
        }
    }
}