using System;
using System.Diagnostics;
using System.Runtime.InteropServices;


namespace Dapfor.Net.Demo.Internal
{
    public static class MemoryHelper
    {
        //Gets count of private bytes, used by the process
        public static long PrivateBytes
        {
            get
            {
                PROCESS_MEMORY_COUNTERS mc = new PROCESS_MEMORY_COUNTERS();
                mc.cb = Marshal.SizeOf(mc);
                using(Process process = Process.GetCurrentProcess())
                {
                    if (GetProcessMemoryInfo(process.Handle, ref mc, mc.cb))
                    {
                        return mc.PagefileUsage;
                    }
                }
                return 0;
            }
        }

        //Gets an amount of virtual memory, used by the process
        public static long VirtualMemory
        {
            get
            {
                MEMORYSTATUS ms = new MEMORYSTATUS();
                if (GlobalMemoryStatus(ref ms))
                {
                    return ms.dwTotalVirtual - ms.dwAvailVirtual;
                }
                return 0;
            }
        }

        #region Private structures

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        private struct MEMORYSTATUS
        {
            public uint dwLength;
            public uint dwMemoryLoad;
            public uint dwTotalPhys;
            public uint dwAvailPhys;
            public uint dwTotalPageFile;
            public uint dwAvailPageFile;
            public uint dwTotalVirtual;
            public uint dwAvailVirtual;
        } // class MEMORYSTATUS

        [StructLayout(LayoutKind.Sequential, Size = 40)]
        private struct PROCESS_MEMORY_COUNTERS
        {
            public int cb;
            public int PageFaultCount;
            public int PeakWorkingSetSize;
            public int WorkingSetSize;
            public int QuotaPeakPagedPoolUsage;
            public int QuotaPagedPoolUsage;
            public int QuotaPeakNonPagedPoolUsage;
            public int QuotaNonPagedPoolUsage;
            public int PagefileUsage;
            public int PeakPagefileUsage;
        }

        #endregion

        #region Private methods

        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool GlobalMemoryStatus(ref MEMORYSTATUS lpBuffer);

        [DllImport("psapi.dll", SetLastError = true)]
        static extern bool GetProcessMemoryInfo(IntPtr hProcess, ref PROCESS_MEMORY_COUNTERS counters, int size);

        #endregion
    }
}